@@ -294,6 +294,7 @@ public class AccessControlList implements Writable {
   /**
    * Returns the access control list as a String that can be used for building a
    * new instance by sending it to the constructor of {@link AccessControlList}.
+   * @return acl string.
    */
   public String getAclString() {
     StringBuilder sb = new StringBuilder(INITIAL_CAPACITY);
@@ -301,9 +302,9 @@ public class AccessControlList implements Writable {
       sb.append('*');
     }
     else {
-      sb.append(getUsersString());
-      sb.append(" ");
-      sb.append(getGroupsString());
+      sb.append(getUsersString())
+          .append(" ")
+          .append(getGroupsString());
     }
     return sb.toString();
   }
