@@ -28,12 +28,11 @@ import org.apache.hadoop.classification.InterfaceAudience.Private;
 import org.apache.hadoop.classification.InterfaceStability;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.CommonConfigurationKeys;
-import org.apache.hadoop.security.KerberosInfo;
 import org.apache.hadoop.security.SecurityUtil;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.util.MachineList;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -97,25 +96,24 @@ public class ServiceAuthorizationManager {
       throw new AuthorizationException("Protocol " + protocol + 
                                        " is not known.");
     }
-    
-    // get client principal key to verify (if available)
+
     String clientPrincipal = null;
-
-
-    try {
+    if (UserGroupInformation.isSecurityEnabled()) {
+      // get client principal key to verify (if available)
       clientPrincipal = SecurityUtil.getClientPrincipal(protocol, conf);
-      if (clientPrincipal != null) {
-
-        clientPrincipal = SecurityUtil.getServerPrincipal(clientPrincipal,
-            addr);
+      try {
+        if (clientPrincipal != null) {
+          clientPrincipal =
+              SecurityUtil.getServerPrincipal(clientPrincipal, addr);
+        }
+      } catch (IOException e) {
+        throw (AuthorizationException) new AuthorizationException(
+            "Can't figure out Kerberos principal name for connection from "
+                + addr + " for user=" + user + " protocol=" + protocol)
+            .initCause(e);
       }
-    } catch (IOException e) {
-      throw (AuthorizationException) new AuthorizationException(
-          "Can't figure out Kerberos principal name for connection from "
-              + addr + " for user=" + user + " protocol=" + protocol)
-          .initCause(e);
+
     }
-    
     if((clientPrincipal != null && !clientPrincipal.equals(user.getUserName())) || 
        acls.length != 2  || !acls[0].isUserAllowed(user) || acls[1].isUserAllowed(user)) {
       String cause = clientPrincipal != null ?
