@@ -58,6 +58,9 @@ class JournalMetrics {
 
   @Metric("Number of batches written where this node was lagging")
   MutableCounterLong batchesWrittenWhileLagging;
+
+  @Metric("Number of edit logs downloaded by JournalNodeSyncer")
+  private MutableCounterLong numEditLogsSynced;
   
   private final int[] QUANTILE_INTERVALS = new int[] {
       1*60, // 1m
@@ -139,6 +142,14 @@ class JournalMetrics {
     }
   }
 
+  public MutableCounterLong getNumEditLogsSynced() {
+    return numEditLogsSynced;
+  }
+
+  public void incrNumEditLogsSynced() {
+    numEditLogsSynced.incr();
+  }
+
   public void addRpcRequestCacheMissAmount(long cacheMissAmount) {
     rpcRequestCacheMissAmount.add(cacheMissAmount);
   }
