@@ -20,7 +20,6 @@ package org.apache.hadoop.hdfs;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
-import java.io.DataOutputStream;
 import java.io.EOFException;
 import java.io.IOException;
 import java.util.ArrayList;
@@ -59,7 +58,7 @@ import org.mockito.Mockito;
 import org.mockito.invocation.InvocationOnMock;
 import org.mockito.stubbing.Answer;
 
-import com.google.common.base.Supplier;
+import java.util.function.Supplier;
 import org.slf4j.LoggerFactory;
 import org.slf4j.Logger;
 
@@ -86,11 +85,9 @@ public class TestPread {
 
   private void writeFile(FileSystem fileSys, Path name) throws IOException {
     int replication = 3;// We need > 1 blocks to test out the hedged reads.
-    // create and write a file that contains three blocks of data
-    DataOutputStream stm = fileSys.create(name, true, 4096,
-      (short)replication, blockSize);
     // test empty file open and read
-    stm.close();
+    DFSTestUtil.createFile(fileSys, name, fileSize, 0,
+      blockSize, (short)replication, seed);
     FSDataInputStream in = fileSys.open(name);
     byte[] buffer = new byte[fileSize];
     in.readFully(0, buffer, 0, 0);
@@ -366,8 +363,8 @@ public class TestPread {
       assertTrue(false);
     } finally {
       Mockito.reset(injector);
-      IOUtils.cleanup(null, input);
-      IOUtils.cleanup(null, output);
+      IOUtils.cleanupWithLogger(null, input);
+      IOUtils.cleanupWithLogger(null, output);
       fileSys.close();
       cluster.shutdown();
     }
@@ -404,9 +401,9 @@ public class TestPread {
     DFSClient dfsClient = fileSys.getClient();
     DFSHedgedReadMetrics metrics = dfsClient.getHedgedReadMetrics();
     // Metrics instance is static, so we need to reset counts from prior tests.
-    metrics.hedgedReadOps.set(0);
-    metrics.hedgedReadOpsWin.set(0);
-    metrics.hedgedReadOpsInCurThread.set(0);
+    metrics.hedgedReadOps.reset();
+    metrics.hedgedReadOpsWin.reset();
+    metrics.hedgedReadOpsInCurThread.reset();
 
     try {
       Path file1 = new Path("hedgedReadMaxOut.dat");
@@ -593,7 +590,7 @@ public class TestPread {
     String filename = "/hedgedReadMaxOut.dat";
     DFSHedgedReadMetrics metrics = dfsClient.getHedgedReadMetrics();
     // Metrics instance is static, so we need to reset counts from prior tests.
-    metrics.hedgedReadOps.set(0);
+    metrics.hedgedReadOps.reset();
     try {
       Path file = new Path(filename);
       output = fileSys.create(file, (short) 2);
@@ -676,14 +673,13 @@ public class TestPread {
     try (MiniDFSCluster cluster =
         new MiniDFSCluster.Builder(conf).numDataNodes(3).build()) {
       DistributedFileSystem dfs = cluster.getFileSystem();
-      final Path p = new Path("/test");
+      Path p = new Path("/test");
       String data = "testingmissingblock";
       DFSTestUtil.writeFile(dfs, p, data);
-
+      DFSTestUtil.waitForReplication(dfs, p, (short) 2, 10000);
       FSDataInputStream in = dfs.open(p);
       List<LocatedBlock> blocks = DFSTestUtil.getAllBlocks(in);
       LocatedBlock lb = blocks.get(0);
-      DFSTestUtil.waitForReplication(cluster, lb.getBlock(), 1, 2, 0);
       blocks = DFSTestUtil.getAllBlocks(in);
       DatanodeInfo[] locations = null;
       for (LocatedBlock locatedBlock : blocks) {
@@ -693,7 +689,7 @@ public class TestPread {
       }
       final DatanodeInfo validDownLocation = locations[0];
       final DFSClient client = dfs.getClient();
-      final DFSClient dfsClient = Mockito.spy(client);
+      DFSClient dfsClient = Mockito.spy(client);
       // Keep the valid location as last in the locations list for second
       // requests
       // onwards.
