@@ -65,8 +65,8 @@ import org.slf4j.event.Level;
 import org.junit.Assert;
 import org.junit.Test;
 
-import com.google.common.base.Preconditions;
-import com.google.common.collect.Maps;
+import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
+import org.apache.hadoop.thirdparty.com.google.common.collect.Maps;
 
 /**
  * Test the data migration tool (for Archival Storage)
@@ -230,7 +230,7 @@ public class TestStorageMover {
     }
 
     void shutdownCluster() throws Exception {
-      IOUtils.cleanup(null, dfs);
+      IOUtils.cleanupWithLogger(null, dfs);
       if (cluster != null) {
         cluster.shutdown();
       }
