@@ -17,9 +17,9 @@
  */
 package org.apache.hadoop.hdfs.server.namenode.ha;
 
-import com.google.common.base.Joiner;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.hadoop.thirdparty.com.google.common.base.Joiner;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.AbstractFileSystem;
 import org.apache.hadoop.fs.CommonConfigurationKeysPublic;
@@ -44,10 +44,10 @@ import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.security.token.Token;
 import org.apache.hadoop.security.token.TokenIdentifier;
 import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.hadoop.test.Whitebox;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
-import org.mockito.internal.util.reflection.Whitebox;
 import org.slf4j.event.Level;
 
 import java.io.ByteArrayInputStream;
@@ -68,8 +68,8 @@ import static org.junit.Assert.*;
  **/
 public class TestDelegationTokensWithHA {
   private static final Configuration conf = new Configuration();
-  private static final Log LOG =
-    LogFactory.getLog(TestDelegationTokensWithHA.class);
+  private static final Logger LOG =
+      LoggerFactory.getLogger(TestDelegationTokensWithHA.class);
   private static MiniDFSCluster cluster;
   private static NameNode nn0;
   private static NameNode nn1;
@@ -95,7 +95,7 @@ public class TestDelegationTokensWithHA {
     cluster.waitActive();
     
     String logicalName = HATestUtil.getLogicalHostname(cluster);
-    HATestUtil.setFailoverConfigurations(cluster, conf, logicalName, 0);
+    HATestUtil.setFailoverConfigurations(cluster, conf, logicalName, null, 0);
 
     nn0 = cluster.getNameNode(0);
     nn1 = cluster.getNameNode(1);
@@ -139,12 +139,8 @@ public class TestDelegationTokensWithHA {
         getDelegationToken(dfs, ugi.getShortUserName());
     ugi.addToken(token);
     // Recreate the DFS, this time authenticating using a DT
-    dfs = ugi.doAs(new PrivilegedExceptionAction<DistributedFileSystem>() {
-      @Override
-      public DistributedFileSystem run() throws Exception {
-        return (DistributedFileSystem) FileSystem.get(conf);
-      }
-    });
+    dfs = ugi.doAs((PrivilegedExceptionAction<DistributedFileSystem>)
+        () -> (DistributedFileSystem) FileSystem.get(conf));
 
     GenericTestUtils.setLogLevel(ObserverReadProxyProvider.LOG, Level.DEBUG);
     GenericTestUtils.LogCapturer logCapture = GenericTestUtils.LogCapturer
