@@ -35,8 +35,6 @@ import java.util.Set;
 import javax.crypto.SecretKey;
 import javax.net.ssl.HttpsURLConnection;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.io.IOUtils;
 import org.apache.hadoop.mapred.Counters;
 import org.apache.hadoop.mapred.JobConf;
@@ -50,12 +48,14 @@ import org.apache.hadoop.mapreduce.CryptoUtils;
 import org.apache.hadoop.security.ssl.SSLFactory;
 import org.apache.hadoop.util.Time;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
 
 class Fetcher<K,V> extends Thread {
   
-  private static final Log LOG = LogFactory.getLog(Fetcher.class);
+  private static final Logger LOG = LoggerFactory.getLogger(Fetcher.class);
   
   /** Number of ms before timing out a copy */
   private static final int DEFAULT_STALLED_COPY_TIMEOUT = 3 * 60 * 1000;
@@ -72,7 +72,7 @@ class Fetcher<K,V> extends Thread {
   private static final String FETCH_RETRY_AFTER_HEADER = "Retry-After";
 
   protected final Reporter reporter;
-  private static enum ShuffleErrors{IO_ERROR, WRONG_LENGTH, BAD_ID, WRONG_MAP,
+  private enum ShuffleErrors{IO_ERROR, WRONG_LENGTH, BAD_ID, WRONG_MAP,
                                     CONNECTION, WRONG_REDUCE}
   
   private final static String SHUFFLE_ERR_GRP_NAME = "Shuffle Errors";
@@ -342,7 +342,7 @@ class Fetcher<K,V> extends Thread {
         try {
           failedTasks = copyMapOutput(host, input, remaining, fetchRetryEnabled);
         } catch (IOException e) {
-          IOUtils.cleanup(LOG, input);
+          IOUtils.cleanupWithLogger(LOG, input);
           //
           // Setup connection again if disconnected by NM
           connection.disconnect();
@@ -372,7 +372,7 @@ class Fetcher<K,V> extends Thread {
       input = null;
     } finally {
       if (input != null) {
-        IOUtils.cleanup(LOG, input);
+        IOUtils.cleanupWithLogger(LOG, input);
         input = null;
       }
       for (TaskAttemptID left : remaining) {
