@@ -39,6 +39,7 @@ import org.slf4j.LoggerFactory;
 import java.io.IOException;
 import java.util.Random;
 
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.Assert.assertEquals;
 
 public class TestMRSequenceFileInputFilter {
@@ -166,7 +167,7 @@ public class TestMRSequenceFileInputFilter {
       int expectedCount = length / 1000;
       if (expectedCount * 1000 != length)
         expectedCount++;
-      assertEquals(count, expectedCount);
+      assertThat(count).isEqualTo(expectedCount);
     }
       
     // clean up
