@@ -105,10 +105,15 @@ public class StagingTestBase {
   /** The raw bucket URI Path before any canonicalization. */
   public static final URI RAW_BUCKET_URI =
       RAW_BUCKET_PATH.toUri();
-  public static Path outputPath =
+
+  @SuppressWarnings("StaticNonFinalField")
+  private static Path outputPath =
       new Path("s3a://" + BUCKET + "/" + OUTPUT_PREFIX);
-  public static URI outputPathUri = outputPath.toUri();
-  public static Path root;
+
+  @SuppressWarnings("StaticNonFinalField")
+  private static URI outputPathUri = getOutputPath().toUri();
+  @SuppressWarnings("StaticNonFinalField")
+  private static Path root;
 
   protected StagingTestBase() {
   }
@@ -131,8 +136,8 @@ public class StagingTestBase {
     URI uri = RAW_BUCKET_URI;
     wrapperFS.initialize(uri, conf);
     root = wrapperFS.makeQualified(new Path("/"));
-    outputPath = new Path(root, OUTPUT_PREFIX);
-    outputPathUri = outputPath.toUri();
+    outputPath = new Path(getRoot(), OUTPUT_PREFIX);
+    outputPathUri = getOutputPath().toUri();
     FileSystemTestHelper.addFileSystemForTesting(uri, conf, wrapperFS);
     return mockFs;
   }
@@ -154,7 +159,7 @@ public class StagingTestBase {
    */
   public static MockS3AFileSystem lookupWrapperFS(Configuration conf)
       throws IOException {
-    return (MockS3AFileSystem) FileSystem.get(outputPathUri, conf);
+    return (MockS3AFileSystem) FileSystem.get(getOutputPathUri(), conf);
   }
 
   public static void verifyCompletion(FileSystem mockS3) throws IOException {
@@ -169,13 +174,13 @@ public class StagingTestBase {
 
   public static void verifyDeleted(FileSystem mockS3, String child)
       throws IOException {
-    verifyDeleted(mockS3, new Path(outputPath, child));
+    verifyDeleted(mockS3, new Path(getOutputPath(), child));
   }
 
   public static void verifyCleanupTempFiles(FileSystem mockS3)
       throws IOException {
     verifyDeleted(mockS3,
-        new Path(outputPath, CommitConstants.TEMPORARY));
+        new Path(getOutputPath(), CommitConstants.TEMPORARY));
   }
 
   protected static void assertConflictResolution(
@@ -189,7 +194,7 @@ public class StagingTestBase {
   public static void pathsExist(FileSystem mockS3, String... children)
       throws IOException {
     for (String child : children) {
-      pathExists(mockS3, new Path(outputPath, child));
+      pathExists(mockS3, new Path(getOutputPath(), child));
     }
   }
 
@@ -231,7 +236,7 @@ public class StagingTestBase {
   public static void canDelete(FileSystem mockS3, String... children)
       throws IOException {
     for (String child : children) {
-      canDelete(mockS3, new Path(outputPath, child));
+      canDelete(mockS3, new Path(getOutputPath(), child));
     }
   }
 
@@ -243,7 +248,7 @@ public class StagingTestBase {
 
   public static void verifyExistenceChecked(FileSystem mockS3, String child)
       throws IOException {
-    verifyExistenceChecked(mockS3, new Path(outputPath, child));
+    verifyExistenceChecked(mockS3, new Path(getOutputPath(), child));
   }
 
   public static void verifyExistenceChecked(FileSystem mockS3, Path path)
@@ -262,6 +267,18 @@ public class StagingTestBase {
     verify(mockS3).mkdirs(path);
   }
 
+  protected static URI getOutputPathUri() {
+    return outputPathUri;
+  }
+
+  static Path getRoot() {
+    return root;
+  }
+
+  static Path getOutputPath() {
+    return outputPath;
+  }
+
   /**
    * Provides setup/teardown of a MiniDFSCluster for tests that need one.
    */
