@@ -28,9 +28,11 @@ import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.contract.ContractTestUtils;
+import org.apache.hadoop.fs.s3a.test.PublicDatasetTestUtils;
 import org.apache.hadoop.fs.s3a.S3AFileSystem;
 
 import static org.apache.hadoop.fs.s3a.Constants.DIRECTORY_MARKER_POLICY_AUTHORITATIVE;
@@ -307,22 +309,25 @@ public class ITestMarkerTool extends AbstractMarkerToolTest {
   }
 
   /**
-   * Run an audit against the landsat bucket.
+   * Run an audit against a bucket with a large number of objects.
    * <p></p>
    * This tests paging/scale against a larger bucket without
    * worrying about setup costs.
    */
   @Test
-  public void testRunLimitedLandsatAudit() throws Throwable {
-    describe("Audit a few thousand landsat objects");
+  public void testRunAuditManyObjectsInBucket() throws Throwable {
+    describe("Audit a few thousand objects");
     final File audit = tempAuditFile();
 
+    Configuration conf = super.createConfiguration();
+    String bucketUri = PublicDatasetTestUtils.getBucketPrefixWithManyObjects(conf);
+
     runToFailure(EXIT_INTERRUPTED,
         MARKERS,
         AUDIT,
         m(OPT_LIMIT), 3000,
         m(OPT_OUT), audit,
-        LANDSAT_BUCKET);
+        bucketUri);
     readOutput(audit);
   }
 
