@@ -27,9 +27,13 @@ import java.util.List;
 import java.util.Set;
 import java.util.Timer;
 import java.util.TimerTask;
+
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.service.AbstractService;
 import org.apache.hadoop.util.DiskChecker.DiskErrorException;
 import org.apache.hadoop.util.DiskValidator;
 import org.apache.hadoop.util.DiskValidatorFactory;
+import org.apache.hadoop.yarn.server.nodemanager.health.HealthReporter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -40,7 +44,6 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.LocalDirAllocator;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.permission.FsPermission;
-import org.apache.hadoop.service.AbstractService;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.exceptions.YarnRuntimeException;
@@ -52,7 +55,8 @@ import org.apache.hadoop.yarn.server.nodemanager.metrics.NodeManagerMetrics;
  * directories of a node. This specifically manages nodemanager-local-dirs and
  * nodemanager-log-dirs by periodically checking their health.
  */
-public class LocalDirsHandlerService extends AbstractService {
+public class LocalDirsHandlerService extends AbstractService
+    implements HealthReporter {
 
   private static final Logger LOG =
        LoggerFactory.getLogger(LocalDirsHandlerService.class);
@@ -424,6 +428,11 @@ public class LocalDirsHandlerService extends AbstractService {
 
   }
 
+  @Override
+  public String getHealthReport() {
+    return getDisksHealthReport(false);
+  }
+
   /**
    * The minimum fraction of number of disks needed to be healthy for a node to
    * be considered healthy in terms of disks is configured using
@@ -455,10 +464,20 @@ public class LocalDirsHandlerService extends AbstractService {
     return true;
   }
 
+  @Override
+  public boolean isHealthy() {
+    return areDisksHealthy();
+  }
+
   public long getLastDisksCheckTime() {
     return lastDisksCheckTime;
   }
 
+  @Override
+  public long getLastHealthReportTime() {
+    return getLastDisksCheckTime();
+  }
+
   public boolean isGoodLocalDir(String path) {
     return isInGoodDirs(getLocalDirs(), path);
   }
@@ -508,7 +527,8 @@ public class LocalDirsHandlerService extends AbstractService {
 
   }
 
-  private void checkDirs() {
+  @VisibleForTesting
+  public void checkDirs() {
     boolean disksStatusChange = false;
     Set<String> failedLocalDirsPreCheck =
         new HashSet<String>(localDirs.getFailedDirs());
