@@ -69,6 +69,9 @@ public final class RMWSConsts {
   /** Path for {@code RMWebServiceProtocol#getNode}. */
   public static final String NODES_NODEID = "/nodes/{nodeId}";
 
+  /** Path for {@code RMWebServiceProtocol#updateNodeResource}. */
+  public static final String NODE_RESOURCE = "/nodes/{nodeId}/resource";
+
   /**
    * Path for {@code RMWebServiceProtocol#getApps} and
    * {@code RMWebServiceProtocol#getApp}.
@@ -80,7 +83,7 @@ public final class RMWSConsts {
 
   /** Path for {@code RMWebServiceProtocol#getAppActivities}. */
   public static final String SCHEDULER_APP_ACTIVITIES =
-      "/scheduler/app-activities";
+      "/scheduler/app-activities/{appid}";
 
   /** Path for {@code RMWebServiceProtocol#getAppStatistics}. */
   public static final String APP_STATISTICS = "/appstatistics";
@@ -198,6 +201,12 @@ public final class RMWSConsts {
   public static final String CHECK_USER_ACCESS_TO_QUEUE =
       "/queues/{queue}/access";
 
+  /**
+   * Path for {@code RMWebServiceProtocol#signalContainer}.
+   */
+  public static final String SIGNAL_TO_CONTAINER =
+      "/containers/{containerid}/signal/{command}";
+
   // ----------------QueryParams for RMWebServiceProtocol----------------
 
   public static final String TIME = "time";
@@ -235,9 +244,34 @@ public final class RMWSConsts {
   public static final String DESELECTS = "deSelects";
   public static final String CONTAINERS = "containers";
   public static final String QUEUE_ACL_TYPE = "queue-acl-type";
+  public static final String REQUEST_PRIORITIES = "requestPriorities";
+  public static final String ALLOCATION_REQUEST_IDS = "allocationRequestIds";
+  public static final String GROUP_BY = "groupBy";
+  public static final String SIGNAL = "signal";
+  public static final String COMMAND = "command";
+  public static final String ACTIONS = "actions";
+  public static final String SUMMARIZE = "summarize";
+  public static final String NAME = "name";
 
   private RMWSConsts() {
     // not called
   }
 
+  /**
+   * Defines the groupBy types of activities, currently only support
+   * DIAGNOSTIC with which user can query aggregated activities
+   * grouped by allocation state and diagnostic.
+   */
+  public enum ActivitiesGroupBy {
+    DIAGNOSTIC
+  }
+
+  /**
+   * Defines the required action of app activities:
+   * REFRESH means to turn on activities recording for the required app,
+   * GET means the required app activities should be involved in response.
+   */
+  public enum AppActivitiesRequiredAction {
+    REFRESH, GET
+  }
 }
\ No newline at end of file
