@@ -27,8 +27,8 @@ import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.server.resourcemanager.RMContext;
 import org.apache.hadoop.yarn.server.resourcemanager.ResourceManager;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.NodesPage.NodesBlock;
+import org.apache.hadoop.yarn.util.resource.CustomResourceTypesConfigurationProvider;
 import org.apache.hadoop.yarn.util.resource.ResourceUtils;
-import org.apache.hadoop.yarn.util.resource.TestResourceUtils;
 import org.apache.hadoop.yarn.webapp.test.WebAppTests;
 import org.junit.Before;
 import org.junit.Test;
@@ -115,7 +115,8 @@ public class TestNodesPage {
       throws Exception {
     Map<String, ResourceInformation> oldRtMap =
         ResourceUtils.getResourceTypes();
-    TestResourceUtils.addNewTypesToResources(ResourceInformation.GPU_URI);
+    CustomResourceTypesConfigurationProvider.
+        initResourceTypes(ResourceInformation.GPU_URI);
     this.setUpInternal(true);
     try {
       // Test gpu as a custom resource.
