@@ -58,7 +58,7 @@ import org.apache.hadoop.yarn.server.timelineservice.storage.common.RowKeyPrefix
 import org.apache.hadoop.yarn.server.timelineservice.storage.common.TimelineStorageUtils;
 import org.apache.hadoop.yarn.webapp.BadRequestException;
 
-import com.google.common.base.Preconditions;
+import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
 
 /**
  * Timeline entity reader for application entities that are stored in the
