@@ -57,7 +57,7 @@ import org.apache.hadoop.yarn.server.timelineservice.reader.TimelineReaderContex
 import org.apache.hadoop.yarn.server.timelineservice.storage.common.TimelineStorageUtils;
 import org.apache.hadoop.yarn.webapp.YarnJacksonJaxbJsonProvider;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -93,7 +93,7 @@ public class FileSystemTimelineReaderImpl extends AbstractService
   private static final String STORAGE_DIR_ROOT = "timeline_service_data";
 
   private final CSVFormat csvFormat =
-      CSVFormat.DEFAULT.withHeader("APP", "USER", "FLOW", "FLOWRUN");
+      CSVFormat.Builder.create().setHeader("APP", "USER", "FLOW", "FLOWRUN").build();
 
   public FileSystemTimelineReaderImpl() {
     super(FileSystemTimelineReaderImpl.class.getName());
@@ -432,6 +432,10 @@ public class FileSystemTimelineReaderImpl extends AbstractService
     String flowRunPathStr = getFlowRunPath(context.getUserId(),
         context.getClusterId(), context.getFlowName(), context.getFlowRunId(),
         context.getAppId());
+    if (context.getUserId() == null) {
+      context.setUserId(new Path(flowRunPathStr).getParent().getParent().
+          getName());
+    }
     Path clusterIdPath = new Path(entitiesPath, context.getClusterId());
     Path flowRunPath = new Path(clusterIdPath, flowRunPathStr);
     Path appIdPath = new Path(flowRunPath, context.getAppId());
@@ -444,14 +448,17 @@ public class FileSystemTimelineReaderImpl extends AbstractService
     return result;
   }
 
-  @Override public TimelineHealth getHealthStatus() {
+  @Override
+  public TimelineHealth getHealthStatus() {
     try {
       fs.exists(rootPath);
     } catch (IOException e) {
       return new TimelineHealth(
           TimelineHealth.TimelineHealthStatus.READER_CONNECTION_FAILURE,
-          e.getMessage());
+          e.getMessage()
+          );
     }
-    return new TimelineHealth(TimelineHealth.TimelineHealthStatus.RUNNING, "");
+    return new TimelineHealth(TimelineHealth.TimelineHealthStatus.RUNNING,
+        "");
   }
 }
\ No newline at end of file
