@@ -20,31 +20,46 @@
 
 import Ember from 'ember';
 
-function getYarnHttpProtocolScheme(rmhost, application) {
+function getConfigFromYarn(rmhost, application, config) {
   var httpUrl = window.location.protocol + '//' +
     (ENV.hosts.localBaseAddress? ENV.hosts.localBaseAddress + '/' : '') + rmhost;
 
-  httpUrl += '/conf?name=yarn.http.policy';
-  Ember.Logger.log("yarn.http.policy URL is: " + httpUrl);
+  httpUrl += '/conf?name=' + config;
+  Ember.Logger.log("The RM URL is: " + httpUrl);
 
-  var protocolScheme = "";
-  $.ajax({
-    type: 'GET',
-    dataType: 'json',
-    async: false,
-    context: this,
-    url: httpUrl,
-    success: function(data) {
-      protocolScheme = data.property.value;
-      Ember.Logger.log("Protocol scheme from RM: " + protocolScheme);
+  var configValue = "";
+    $.ajax({
+      type: 'GET',
+      dataType: 'json',
+      async: false,
+      context: this,
+      url: httpUrl,
+      success: function(data) {
+        configValue = data.property.value;
+        Ember.Logger.log("Value of the config returned from RM: " + configValue);
 
-      application.advanceReadiness();
-    },
-    error: function() {
-      application.advanceReadiness();
-    }
-  });
-  return protocolScheme;
+        application.advanceReadiness();
+      },
+      error: function() {
+        application.advanceReadiness();
+      }
+    });
+  return configValue;
+}
+
+function getJHSURL(rmhost, application, isHttpsSchemeEnabled) {
+  Ember.Logger.log("getJHSURL, params:rmhost=" + rmhost + ",application=" + application + ",isHttpsSchemeEnabled=" + isHttpsSchemeEnabled);
+  var config = '';
+  if (isHttpsSchemeEnabled) {
+    config = 'mapreduce.jobhistory.webapp.https.address';
+  } else {
+    config = 'mapreduce.jobhistory.webapp.address';
+  }
+  return getConfigFromYarn(rmhost, application, config);
+}
+
+function getYarnHttpProtocolScheme(rmhost, application) {
+  return getConfigFromYarn(rmhost, application, 'yarn.http.policy');
 }
 
 function getTimeLineURL(rmhost, isHttpsSchemeEnabled) {
@@ -84,9 +99,75 @@ function getSecurityURL(rmhost) {
   return url;
 }
 
+function getClusterIdFromYARN(rmhost, application) {
+  var httpUrl = window.location.protocol + '//' +
+    (ENV.hosts.localBaseAddress? ENV.hosts.localBaseAddress + '/' : '') + rmhost;
+
+  httpUrl += '/conf?name=yarn.resourcemanager.cluster-id';
+  Ember.Logger.log("Get cluster-id URL is: " + httpUrl);
+
+  var clusterId = "";
+  $.ajax({
+    type: 'GET',
+    dataType: 'json',
+    async: false,
+    context: this,
+    url: httpUrl,
+    success: function(data) {
+      clusterId = data.property.value;
+      Ember.Logger.log("Cluster Id from RM: " + clusterId);
+      application.advanceReadiness();
+    },
+    error: function() {
+      application.advanceReadiness();
+    }
+  });
+  return clusterId;
+}
+
+function getNodeManagerPort(rmhost, application) {
+  var httpUrl = window.location.protocol + "//" +
+    (ENV.hosts.localBaseAddress ? ENV.hosts.localBaseAddress + '/' : '') + rmhost
+    + "/conf?name=yarn.nodemanager.webapp.address";
+
+  var port = "8042";
+  $.ajax({
+    type: 'GET',
+    dataType: 'json',
+    async: false,
+    context: this,
+    url: httpUrl,
+    success: function(data) {
+      port = data.property.value.split(":")[1];
+      application.advanceReadiness();
+    },
+    error: function() {
+      port = "8042";
+      application.advanceReadiness();
+    }
+  });
+  return port;
+}
+
+function transformURL(url, hostname) {
+  // Deleting the scheme from the beginning of the url
+  url = url.replace(/(^\w+:|^)\/\//, '');
+
+  var address = url.split(":")[0];
+  var port = url.split(":")[1];
+  // Instead of localhost, use the name of the host
+  if (address === "0.0.0.0" || address === "localhost") {
+    url = hostname + ":" + port;
+  }
+
+  Ember.Logger.log("The transformed URL is: " + url);
+  return url;
+}
+
 function updateConfigs(application) {
   var hostname = window.location.hostname;
-  var rmhost = hostname + (window.location.port ? ':' + window.location.port: '') + skipTrailingSlash(window.location.pathname);
+  var rmhost = hostname + (window.location.port ? ':' + window.location.port: '') +
+    skipTrailingSlash(window.location.pathname);
 
   window.ENV = window.ENV || {};
   window.ENV.hosts = window.ENV.hosts || {};
@@ -103,6 +184,21 @@ function updateConfigs(application) {
   var protocolSchemeFromRM = getYarnHttpProtocolScheme(rmhost, application);
   Ember.Logger.log("Is protocol scheme https? " + (protocolSchemeFromRM == "HTTPS_ONLY"));
   var isHttpsSchemeEnabled = (protocolSchemeFromRM == "HTTPS_ONLY");
+
+  var clusterIdFromYARN = getClusterIdFromYARN(rmhost, application);
+  ENV.clusterId = clusterIdFromYARN;
+
+  var nodeManagerPort = getNodeManagerPort(rmhost, application);
+  Ember.Logger.log("NodeMananger port: " + nodeManagerPort);
+  ENV.nodeManagerPort = nodeManagerPort;
+
+  if (!ENV.hosts.jhsAddress) {
+    var jhsAddress = getJHSURL(rmhost, application, isHttpsSchemeEnabled);
+    jhsAddress = transformURL(jhsAddress, hostname);
+    Ember.Logger.log("The JHS address is " + jhsAddress);
+    ENV.hosts.jhsAddress = jhsAddress;
+  }
+
   if(!ENV.hosts.timelineWebAddress) {
     var timelinehost = "";
     $.ajax({
@@ -113,19 +209,10 @@ function updateConfigs(application) {
       url: getTimeLineURL(rmhost, isHttpsSchemeEnabled),
       success: function(data) {
         timelinehost = data.property.value;
-        timelinehost = timelinehost.replace(/(^\w+:|^)\/\//, '');
+        timelinehost = transformURL(timelinehost, hostname);
         ENV.hosts.timelineWebAddress = timelinehost;
-
-        var address = timelinehost.split(":")[0];
-        var port = timelinehost.split(":")[1];
-
         Ember.Logger.log("Timeline Address from RM: " + timelinehost);
 
-        if(address === "0.0.0.0" || address === "localhost") {
-          var updatedAddress =  hostname + ":" + port;
-          ENV.hosts.timelineWebAddress = updatedAddress;
-          Ember.Logger.log("Timeline Updated Address: " + updatedAddress);
-        }
         application.advanceReadiness();
       },
       error: function() {
@@ -147,19 +234,10 @@ function updateConfigs(application) {
       url: getTimeLineV1URL(rmhost, isHttpsSchemeEnabled),
       success: function(data) {
         timelinehost = data.property.value;
-        timelinehost = timelinehost.replace(/(^\w+:|^)\/\//, '');
+        timelinehost = transformURL(timelinehost, hostname);
         ENV.hosts.timelineV1WebAddress = timelinehost;
-
-        var address = timelinehost.split(":")[0];
-        var port = timelinehost.split(":")[1];
-
         Ember.Logger.log("Timeline V1 Address from RM: " + timelinehost);
 
-        if(address === "0.0.0.0" || address === "localhost") {
-          var updatedAddress =  hostname + ":" + port;
-          ENV.hosts.timelineV1WebAddress = updatedAddress;
-          Ember.Logger.log("Timeline V1 Updated Address: " + updatedAddress);
-        }
         application.advanceReadiness();
       },
       error: function() {
@@ -207,6 +285,7 @@ export default {
 };
 
 const skipTrailingSlash = function(path) {
+  path = path.replace('index.html', '');
   path = path.replace('ui2/', '');
   path = path.replace(/\/$/, '');
   console.log('base url:' + path)
