@@ -80,6 +80,10 @@ export default Ember.Service.extend({
     return this.normalizeURL(this.get("env.app.hosts.rmWebAddress"));
   }),
 
+  jhsAddress: Ember.computed(function() {
+    return this.normalizeURL(this.get("env.app.hosts.jhsAddress"));
+  }),
+
   dashWebAddress: Ember.computed(function () {
     return this.normalizeURL(this.get("env.app.hosts.dashWebAddress"));
   }),
